@@ -23,10 +23,12 @@
 """Utility functions for quick and easy display."""
 
 from __future__ import absolute_import
+
 import numpy as np
 
 try:
-    from pyresample import _cartopy  # noqa
+    from pyresample.utils import cartopy  # noqa
+
     BASEMAP_NOT_CARTOPY = False
 except ImportError:
     BASEMAP_NOT_CARTOPY = True
@@ -43,7 +45,6 @@ def ellps2axis(ellps_name):
     Returns
     -------
     (a, b) : semi-major and semi-minor axis
-
     """
     ellps = {'helmert': {'a': 6378200.0, 'b': 6356818.1696278909},
              'intl': {'a': 6378388.0, 'b': 6356911.9461279465},
@@ -110,7 +111,6 @@ def area_def2basemap(area_def, **kwargs):
     Returns
     -------
     bmap : Basemap object
-
     """
     import warnings
     warnings.warn("Basemap is no longer maintained. Please switch to cartopy "
@@ -223,11 +223,11 @@ def _add_gridlines(axes, nmeridians, nparallels):
 
     gl = axes.gridlines()
     if nmeridians:
-        gl.xlocator = mticker.FixedLocator(np.arange(-180, 180+nmeridians, nmeridians))
+        gl.xlocator = mticker.FixedLocator(np.arange(-180, 180 + nmeridians, nmeridians))
     else:
         gl.xlines = False
     if nparallels:
-        gl.ylocator = mticker.FixedLocator(np.arange(-90, 90+nparallels, nparallels))
+        gl.ylocator = mticker.FixedLocator(np.arange(-90, 90 + nparallels, nparallels))
     else:
         gl.ylines = False
 
@@ -292,7 +292,6 @@ def show_quicklook(area_def, data, vmin=None, vmax=None,
     Returns
     -------
     bmap : Basemap object
-
     """
     plt = _get_quicklook(area_def, data, vmin=vmin, vmax=vmax,
                          label=label, num_meridians=num_meridians,
@@ -304,7 +303,7 @@ def show_quicklook(area_def, data, vmin=None, vmax=None,
 
 def save_quicklook(filename, area_def, data, vmin=None, vmax=None,
                    label='Variable (units)', num_meridians=45,
-                   num_parallels=10, coast_res='110m', backend='AGG',
+                   num_parallels=10, coast_res='110m',
                    cmap='RdBu_r'):
     """Display and save default quicklook plot.
 
@@ -328,12 +327,7 @@ def save_quicklook(filename, area_def, data, vmin=None, vmax=None,
         Number of parallels to plot on the globe
     coast_res : {'c', 'l', 'i', 'h', 'f'}, optional
         Resolution of coastlines
-    backend : str, optional
-        matplotlib backend to use'
-
     """
-    import matplotlib
-    matplotlib.use(backend, warn=False)
     plt = _get_quicklook(area_def, data, vmin=vmin, vmax=vmax,
                          label=label, num_meridians=num_meridians,
                          num_parallels=num_parallels, coast_res=coast_res,
