@@ -34,7 +34,11 @@ EPSILON = 0.0000001
 
 
 class Coordinate(object):
-    """Point on earth in terms of lat and lon."""
+    """Point on earth in terms of lat and lon.
+
+    It expects lon,lat in degrees
+    But self.lat and self.lon are returned in radians !
+    """
 
     lat = None
     lon = None
@@ -46,7 +50,7 @@ class Coordinate(object):
                  x__=None, y__=None, z__=None, R__=1):
         self.R__ = R__
         if lat is not None and lon is not None:
-            if not(-180 <= lon <= 180 and -90 <= lat <= 90):
+            if not (-180 <= lon <= 180 and -90 <= lat <= 90):
                 raise ValueError('Illegal (lon, lat) coordinates: (%s, %s)'
                                  % (lon, lat))
             self.lat = math.radians(lat)
@@ -71,8 +75,8 @@ class Coordinate(object):
 
     def __ne__(self, other):
         """Check inequality."""
-        if(abs(self.lat - other.lat) < EPSILON and
-           abs(self.lon - other.lon) < EPSILON):
+        if (abs(self.lat - other.lat) < EPSILON and
+                abs(self.lon - other.lon) < EPSILON):
             return 0
         else:
             return 1
@@ -236,25 +240,30 @@ class Arc(object):
 
     def intersections(self, other_arc):
         """Get the two intersections of the greats circles defined by the current arc and *other_arc*."""
-        if self.end.lon - self.start.lon > math.pi:
-            self.end.lon -= 2 * math.pi
-        if other_arc.end.lon - other_arc.start.lon > math.pi:
-            other_arc.end.lon -= 2 * math.pi
-        if self.end.lon - self.start.lon < -math.pi:
-            self.end.lon += 2 * math.pi
-        if other_arc.end.lon - other_arc.start.lon < -math.pi:
-            other_arc.end.lon += 2 * math.pi
+        end_lon = self.end.lon
+        other_end_lon = other_arc.end.lon
 
-        ea_ = self.start.cross2cart(self.end).normalize()
-        eb_ = other_arc.start.cross2cart(other_arc.end).normalize()
+        if self.end.lon - self.start.lon > math.pi:
+            end_lon -= 2 * math.pi
+        if other_arc.end.lon - other_arc.start.lon > math.pi:
+            other_end_lon -= 2 * math.pi
+        if self.end.lon - self.start.lon < -math.pi:
+            end_lon += 2 * math.pi
+        if other_arc.end.lon - other_arc.start.lon < -math.pi:
+            other_end_lon += 2 * math.pi
+
+        end_point = Coordinate(math.degrees(modpi(end_lon)), math.degrees(self.end.lat))
+        other_end_point = Coordinate(math.degrees(modpi(other_end_lon)), math.degrees(other_arc.end.lat))
+
+        ea_ = self.start.cross2cart(end_point).normalize()
+        eb_ = other_arc.start.cross2cart(other_end_point).normalize()
 
         cross = ea_.cross(eb_)
         lat = math.atan2(cross.z__, math.sqrt(cross.x__ ** 2 + cross.y__ ** 2))
         lon = math.atan2(-cross.y__, cross.x__)
 
         return (Coordinate(math.degrees(lon), math.degrees(lat)),
-                Coordinate(math.degrees(modpi(lon + math.pi)),
-                           math.degrees(-lat)))
+                Coordinate(math.degrees(modpi(lon + math.pi)), math.degrees(-lat)))
 
     def intersects(self, other_arc):
         """Determine if this arc and another arc intersect.
@@ -277,8 +286,8 @@ class Arc(object):
             ab_ = a__.distance(b__)
             cd_ = c__.distance(d__)
 
-            if(abs(a__.distance(i) + b__.distance(i) - ab_) < EPSILON and
-               abs(c__.distance(i) + d__.distance(i) - cd_) < EPSILON):
+            if (abs(a__.distance(i) + b__.distance(i) - ab_) < EPSILON and
+                    abs(c__.distance(i) + d__.distance(i) - cd_) < EPSILON):
                 return i
         return None
 
