@@ -18,15 +18,21 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Utilities for testing.
 
-This mostly takes from astropy's method for checking warnings during tests.
+This mostly takes from astropy's method for checking collected_warnings during
+tests.
 """
 import sys
-import six
 import types
 import warnings
+from contextlib import contextmanager
 
 import numpy as np
 
+try:
+    from pyproj import CRS
+except ImportError:
+    CRS = None
+
 _deprecations_as_exceptions = False
 _include_astropy_deprecations = False
 AstropyDeprecationWarning = None
@@ -34,17 +40,14 @@ AstropyPendingDeprecationWarning = None
 
 
 def treat_deprecations_as_exceptions():
-    """
-    Turn all DeprecationWarnings (which indicate deprecated uses of
-    Python itself or Numpy, but not within Astropy, where we use our
-    own deprecation warning class) into exceptions so that we find
-    out about them early.
+    """Turn all DeprecationWarnings into exceptions.
 
+    Deprecation warnings indicate deprecated uses of Python itself or Numpy.
     This completely resets the warning filters and any "already seen"
     warning state.
     """
     # First, totally reset the warning state
-    for module in list(six.itervalues(sys.modules)):
+    for module in sys.modules.values():
         # We don't want to deal with six.MovedModules, only "real"
         # modules.
         if (isinstance(module, types.ModuleType) and
@@ -84,31 +87,30 @@ def treat_deprecations_as_exceptions():
         warnings.filterwarnings("error", ".*", AstropyDeprecationWarning)
         warnings.filterwarnings("error", ".*", AstropyPendingDeprecationWarning)
 
-    if sys.version_info[:2] >= (3, 4):
-        # py.test reads files with the 'U' flag, which is now
-        # deprecated in Python 3.4.
-        warnings.filterwarnings(
-            "ignore",
-            r"'U' mode is deprecated",
-            DeprecationWarning)
+    # py.test reads files with the 'U' flag, which is now
+    # deprecated in Python 3.4.
+    warnings.filterwarnings(
+        "ignore",
+        r"'U' mode is deprecated",
+        DeprecationWarning)
 
-        # BeautifulSoup4 triggers a DeprecationWarning in stdlib's
-        # html module.x
-        warnings.filterwarnings(
-            "ignore",
-            r"The strict argument and mode are deprecated\.",
-            DeprecationWarning)
-        warnings.filterwarnings(
-            "ignore",
-            r"The value of convert_charrefs will become True in 3\.5\. "
-            r"You are encouraged to set the value explicitly\.",
-            DeprecationWarning)
-        # Filter out pyresample's deprecation warnings.
-        warnings.filterwarnings(
-            "ignore",
-            r"This module will be removed in pyresample 2\.0\, please use the"
-            r"\`pyresample.spherical\` module functions and class instead\.",
-            DeprecationWarning)
+    # BeautifulSoup4 triggers a DeprecationWarning in stdlib's
+    # html module.x
+    warnings.filterwarnings(
+        "ignore",
+        r"The strict argument and mode are deprecated\.",
+        DeprecationWarning)
+    warnings.filterwarnings(
+        "ignore",
+        r"The value of convert_charrefs will become True in 3\.5\. "
+        r"You are encouraged to set the value explicitly\.",
+        DeprecationWarning)
+    # Filter out pyresample's deprecation warnings.
+    warnings.filterwarnings(
+        "ignore",
+        r"This module will be removed in pyresample 2\.0\, please use the"
+        r"\`pyresample.spherical\` module functions and class instead\.",
+        DeprecationWarning)
 
     if sys.version_info[:2] >= (3, 5):
         # py.test raises this warning on Python 3.5.
@@ -122,10 +124,10 @@ def treat_deprecations_as_exceptions():
 
 
 class catch_warnings(warnings.catch_warnings):
-    """
-    A high-powered version of warnings.catch_warnings to use for testing
-    and to make sure that there is no dependence on the order in which
-    the tests are run.
+    """A high-powered version of warnings.catch_warnings to use for testing.
+
+    Makes sure that there is no dependence on the order in which the tests
+    are run.
 
     This completely blitzes any memory of any warnings that have
     appeared before so that all warnings will be caught and displayed.
@@ -139,11 +141,14 @@ class catch_warnings(warnings.catch_warnings):
             do.something.bad()
         assert len(w) > 0
     """
+
     def __init__(self, *classes):
+        """Initialize the classes of warnings to catch."""
         super(catch_warnings, self).__init__(record=True)
         self.classes = classes
 
     def __enter__(self):
+        """Catch any warnings during this context."""
         warning_list = super(catch_warnings, self).__enter__()
         treat_deprecations_as_exceptions()
         if len(self.classes) == 0:
@@ -155,14 +160,18 @@ class catch_warnings(warnings.catch_warnings):
         return warning_list
 
     def __exit__(self, type, value, traceback):
+        """Raise any warnings as errors."""
         treat_deprecations_as_exceptions()
 
 
 def create_test_longitude(start, stop, shape, twist_factor=0.0, dtype=np.float32):
-    if start > stop:
+    """Get basic sample of longitude data."""
+    if start > 0 > stop:
+        # cross anti-meridian
         stop += 360.0
 
-    lon_row = np.linspace(start, stop, num=shape[1]).astype(dtype)
+    num_cols = 1 if len(shape) < 2 else shape[1]
+    lon_row = np.linspace(start, stop, num=num_cols).astype(dtype)
     twist_array = np.arange(shape[0]).reshape((shape[0], 1)) * twist_factor
     lon_array = np.repeat([lon_row], shape[0], axis=0)
     lon_array += twist_array
@@ -173,9 +182,11 @@ def create_test_longitude(start, stop, shape, twist_factor=0.0, dtype=np.float32
 
 
 def create_test_latitude(start, stop, shape, twist_factor=0.0, dtype=np.float32):
+    """Get basic sample of latitude data."""
+    num_cols = 1 if len(shape) < 2 else shape[1]
     lat_col = np.linspace(start, stop, num=shape[0]).astype(dtype).reshape((shape[0], 1))
-    twist_array = np.arange(shape[1]) * twist_factor
-    lat_array = np.repeat(lat_col, shape[1], axis=1)
+    twist_array = np.arange(num_cols) * twist_factor
+    lat_array = np.repeat(lat_col, num_cols, axis=1)
     lat_array += twist_array
     return lat_array
 
@@ -196,3 +207,74 @@ class CustomScheduler(object):
             raise RuntimeError("Too many dask computations were scheduled: "
                                "{}".format(self.total_computes))
         return dask.get(dsk, keys, **kwargs)
+
+
+@contextmanager
+def assert_maximum_dask_computes(max_computes=1):
+    """Context manager to make sure dask computations are not executed more than ``max_computes`` times."""
+    import dask
+    with dask.config.set(scheduler=CustomScheduler(max_computes=max_computes)) as new_config:
+        yield new_config
+
+
+def friendly_crs_equal(expected, actual, keys=None, use_obj=True, use_wkt=True):
+    """Test if two projection definitions are equal.
+
+    The main purpose of this function is to help manage differences
+    between pyproj versions. Depending on the version installed and used
+    pyresample may provide a different `proj_dict` or other similar
+    CRS definition.
+
+    Args:
+        expected (dict, str, pyproj.crs.CRS): Expected CRS definition as
+            a PROJ dictionary or string or CRS object.
+        actual (dict, str, pyproj.crs.CRS): Actual CRS definition
+        keys (list): Specific PROJ parameters to look for. Only takes effect
+            if `use_obj` is `False`.
+        use_obj (bool): Use pyproj's CRS object to test equivalence. Default
+            is True.
+        use_wkt (bool): Increase likely hood of making CRS objects equal by
+            converting WellKnownText before converting to the final CRS
+            object. Requires `use_obj`. Defaults to True.
+
+    """
+    if CRS is not None and use_obj:
+        if hasattr(expected, 'crs'):
+            expected = expected.crs
+        if hasattr(actual, 'crs'):
+            actual = actual.crs
+        expected_crs = CRS.from_user_input(expected)
+        actual_crs = CRS.from_user_input(actual)
+        if use_wkt:
+            expected_crs = CRS(expected_crs.to_wkt())
+            actual_crs = CRS(actual_crs.to_wkt())
+        assert expected_crs == actual_crs
+        return
+    raise NotImplementedError("""TODO""")
+
+
+def assert_warnings_contain(collected_warnings: list, message: str, count: int = 1):
+    """Check that collected warnings with catch_warnings contain certain messages.
+
+    Args:
+        collected_warnings:
+            List of warnings collected using
+            ``warnings.catch_warnings(record=True)``.
+        message:
+            Lowercase string to check is in one or more of the warning
+            messages.
+        count:
+            Number of warnings that should contain the provided message
+            string.
+
+    Examples:
+        Use during test code with::
+
+            with warnings.catch_warnings(record=True) as w:
+                # test code
+            assert_warnings_contain(w, "invalid data", 1)
+
+    """
+    msgs = [msg.message.args[0].lower() for msg in collected_warnings]
+    msgs_with = [msg for msg in msgs if message in msg]
+    assert len(msgs_with) == count
