@@ -20,46 +20,60 @@ try:
     from collections.abc import Mapping
 except ImportError:
     from collections import Mapping
-import numpy as np
-import pyproj
+
 import warnings
 
-from ._proj4 import (proj4_dict_to_str, proj4_str_to_dict, convert_proj_floats, proj4_radius_parameters)  # noqa
-from ._rasterio import get_area_def_from_raster  # noqa
+import numpy as np
+
+from .cf import load_cf_area  # noqa
+from .proj4 import (  # noqa
+    convert_proj_floats,
+    get_geostationary_height,
+    proj4_dict_to_str,
+    proj4_radius_parameters,
+    proj4_str_to_dict,
+)
+from .rasterio import get_area_def_from_raster  # noqa
 
 
 def get_area_def(*args, **kwargs):
+    """Get an area definition."""
     from pyresample.area_config import get_area_def
-    warnings.warn("'get_area_def' has moved, import it with 'from pyresample import get_area_def'")
+    warnings.warn("'get_area_def' has moved, import it with 'from pyresample import get_area_def'", stacklevel=2)
     return get_area_def(*args, **kwargs)
 
 
 def create_area_def(*args, **kwargs):
+    """Create an area definition."""
     from pyresample.area_config import create_area_def
-    warnings.warn("'create_area_def' has moved, import it with 'from pyresample import create_area_def'")
+    warnings.warn("'create_area_def' has moved, import it with 'from pyresample import create_area_def'", stacklevel=2)
     return create_area_def(*args, **kwargs)
 
 
 def load_area(*args, **kwargs):
+    """Load an area."""
     from pyresample.area_config import load_area
-    warnings.warn("'load_area' has moved, import it with 'from pyresample import load_area'")
+    warnings.warn("'load_area' has moved, import it with 'from pyresample import load_area'", stacklevel=2)
     return load_area(*args, **kwargs)
 
 
 def convert_def_to_yaml(*args, **kwargs):
+    """Convert an area definition to yaml representation."""
     from pyresample.area_config import convert_def_to_yaml
-    warnings.warn("'convert_def_to_yaml' has moved, import it with 'from pyresample import convert_def_to_yaml'")
+    warnings.warn("'convert_def_to_yaml' has moved, import it with 'from pyresample import convert_def_to_yaml'",
+                  stacklevel=2)
     return convert_def_to_yaml(*args, **kwargs)
 
 
 def parse_area_file(*args, **kwargs):
+    """Parse an area file."""
     from pyresample.area_config import parse_area_file
-    warnings.warn("'parse_area_file' has moved, import it with 'from pyresample import parse_area_file'")
+    warnings.warn("'parse_area_file' has moved, import it with 'from pyresample import parse_area_file'", stacklevel=2)
     return parse_area_file(*args, **kwargs)
 
 
 def generate_quick_linesample_arrays(source_area_def, target_area_def, nprocs=1):
-    """Generate linesample arrays for quick grid resampling
+    """Generate linesample arrays for quick grid resampling.
 
     Parameters
     -----------
@@ -93,7 +107,7 @@ def generate_nearest_neighbour_linesample_arrays(source_area_def,
                                                  target_area_def,
                                                  radius_of_influence,
                                                  nprocs=1):
-    """Generate linesample arrays for nearest neighbour grid resampling
+    """Generate linesample arrays for nearest neighbour grid resampling.
 
     Parameters
     -----------
@@ -110,7 +124,6 @@ def generate_nearest_neighbour_linesample_arrays(source_area_def,
     -------
     (row_indices, col_indices) : tuple of numpy arrays
     """
-
     from pyresample.kd_tree import get_neighbour_info
     valid_input_index, valid_output_index, index_array, distance_array = \
         get_neighbour_info(source_area_def,
@@ -150,7 +163,7 @@ def generate_nearest_neighbour_linesample_arrays(source_area_def,
 
 
 def fwhm2sigma(fwhm):
-    """Calculate sigma for gauss function from FWHM (3 dB level)
+    """Calculate sigma for gauss function from FWHM (3 dB level).
 
     Parameters
     ----------
@@ -161,16 +174,12 @@ def fwhm2sigma(fwhm):
     -------
     sigma : float
         sigma for use in resampling gauss function
-
     """
-
     return fwhm / (2 * np.sqrt(np.log(2)))
 
 
 def _downcast_index_array(index_array, size):
-    """Try to downcast array to uint16
-    """
-
+    """Try to downcast array to uint16."""
     if size <= np.iinfo(np.uint16).max:
         mask = (index_array < 0) | (index_array >= size)
         index_array[mask] = size
@@ -179,7 +188,7 @@ def _downcast_index_array(index_array, size):
 
 
 def wrap_longitudes(lons):
-    """Wrap longitudes to the [-180:+180[ validity range (preserves dtype)
+    """Wrap longitudes to the [-180:+180[ validity range (preserves dtype).
 
     Parameters
     ----------
@@ -190,7 +199,6 @@ def wrap_longitudes(lons):
     -------
     lons : numpy array
         Longitudes wrapped into [-180:+180[ validity range
-
     """
     return (lons + 180) % 360 - 180
 
@@ -208,7 +216,6 @@ def check_and_wrap(lons, lats):
 
     Raises:
         ValueError: If latitude array is not between -90 and 90
-
     """
     # check the latitutes
     if lats.min() < -90. or lats.max() > 90.:
@@ -224,12 +231,11 @@ def check_and_wrap(lons, lats):
 
 
 def recursive_dict_update(d, u):
-    """Recursive dictionary update using
+    """Update dictionary recursively.
 
     Copied from:
 
         http://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth
-
     """
     for k, v in u.items():
         if isinstance(v, Mapping):
@@ -240,6 +246,11 @@ def recursive_dict_update(d, u):
     return d
 
 
-def is_pyproj2():
-    """Determine whether the current pyproj version is >= 2.0"""
-    return pyproj.__version__ >= '2'
+def check_slice_orientation(sli):
+    """Check that the slice is slicing the right way."""
+    if sli.start > sli.stop:
+        if sli.step is None or sli.step > 0:
+            step = -(sli.step or 1)
+            sli = slice(sli.start, sli.stop, step)
+
+    return sli
